/* --------------------------------------------------------------------------------------------
 * Copyright (c) TypeFox and others. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_jsonrpc_1 = require("vscode-jsonrpc");
/**
 * Request to request the call hierarchy at a given text document position.
 *
 * The request's parameter is of type [CallHierarchyParams](#CallHierarchyParams). The response
 * is of type [CallHierarchyItem](#CallHierarchyItem) or a Thenable that resolves to such.
 *
 * The optional request's parameter defines the maximum number of levels to [resolve](#CallHierarchyParams.resolve) by this request.
 * Unresolved items can be resolved in subsequent `callHierarchy/resolve` requests.
 */
var CallHierarchyRequest;
(function (CallHierarchyRequest) {
    CallHierarchyRequest.type = new vscode_jsonrpc_1.RequestType('textDocument/callHierarchy');
})(CallHierarchyRequest = exports.CallHierarchyRequest || (exports.CallHierarchyRequest = {}));
/**
 * Request to resolve a call hierarchy item.
 *
 * The request's parameter is of type [ResolveCallHierarchyItemParams](#ResolveCallHierarchyItemParams). The response
 * is of type [CallHierarchyItem](#CallHierarchyItem) or a Thenable that resolves to such.
 */
var CallHierarchyResolveRequest;
(function (CallHierarchyResolveRequest) {
    CallHierarchyResolveRequest.type = new vscode_jsonrpc_1.RequestType('callHierarchy/resolve');
})(CallHierarchyResolveRequest = exports.CallHierarchyResolveRequest || (exports.CallHierarchyResolveRequest = {}));
/**
 * The direction of a call hierarchy.
 */
var CallHierarchyDirection;
(function (CallHierarchyDirection) {
    /**
     * The callers of a symbol.
     */
    CallHierarchyDirection.Incoming = 1;
    /**
     * The callees of a symbol.
     */
    CallHierarchyDirection.Outgoing = 2;
})(CallHierarchyDirection = exports.CallHierarchyDirection || (exports.CallHierarchyDirection = {}));
//# sourceMappingURL=call-hierarchy.lsp.proposed.js.map